Class: RubyKpi::KP

Inherits:
Object
  • Object
show all
Defined in:
lib/ruby_kpi.rb

Overview

The KP class

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(ip, port, smart_space, debug = false) ⇒ KP

constructor



87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
# File 'lib/ruby_kpi.rb', line 87

def initialize(ip, port, smart_space, debug = false)
  
  # debug object
  @debug = debug
  if @debug       
    @logger = Logger.new(STDOUT)
    @logger.debug("KP:initialize")
  end

  # instance variables
  @ip = ip
  @port = port
  @ss = smart_space
  @transaction_id = 1
  @node_id = UUID.new().generate() 
  @last_request = nil
  @last_reply = nil
  @active_subscriptions = {}

end

Instance Attribute Details

#active_subscriptionsObject (readonly)

accessors



78
79
80
# File 'lib/ruby_kpi.rb', line 78

def active_subscriptions
  @active_subscriptions
end

#ipObject (readonly)

accessors



78
79
80
# File 'lib/ruby_kpi.rb', line 78

def ip
  @ip
end

#last_replyObject (readonly)

accessors



78
79
80
# File 'lib/ruby_kpi.rb', line 78

def last_reply
  @last_reply
end

#last_requestObject (readonly)

accessors



78
79
80
# File 'lib/ruby_kpi.rb', line 78

def last_request
  @last_request
end

#node_idObject (readonly)

accessors



78
79
80
# File 'lib/ruby_kpi.rb', line 78

def node_id
  @node_id
end

#portObject (readonly)

accessors



78
79
80
# File 'lib/ruby_kpi.rb', line 78

def port
  @port
end

#ssObject (readonly)

accessors



78
79
80
# File 'lib/ruby_kpi.rb', line 78

def ss
  @ss
end

Instance Method Details

#insert(triple_list) ⇒ Object

INSERT



204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
250
251
252
# File 'lib/ruby_kpi.rb', line 204

def insert(triple_list)

  # debug print
  if @debug
    @logger.debug("KP:insert")
  end

  # build the triple_string
  triple_string = ""
  triple_list.each do |triple|
    triple_string += TRIPLE_TEMPLATE % [triple.subject.class.to_s.split("::").last.downcase, 
                                        triple.subject.value, 
                                        triple.predicate.value, 
                                        triple.object.class.to_s.split("::").last.downcase, 
                                        triple.object.value]
  end

  # build the SSAP INSERT REQUEST
  msg = INSERT_REQUEST_TEMPLATE % [ @node_id, @ss, @transaction_id, triple_string ]

  # connecting to the SIB
  tcpc = TCPConnection.new(@ip, @port)
  
  # sendind the request
  tcpc.send_request(msg)

  # waiting a reply
  rmsg = tcpc.receive_reply()

  # closing the socket
  tcpc.close()

  # storing last reply
  @last_reply = rmsg

  # storing last request and last reply
  @last_request = msg
  @last_reply = rmsg

  # read the reply
  r = ReplyMessage.new(rmsg)

  # increment transaction id
  @transaction_id += 1

  # return
  return r.success?()
  
end

#join_ssObject

join



115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
# File 'lib/ruby_kpi.rb', line 115

def join_ss()

  # debug print
  if @debug
    @logger.debug("KP:join_ss")
  end

  # building and storing the SSAP JOIN REQUEST
  msg = JOIN_REQUEST_TEMPLATE % [ @node_id, @ss, @transaction_id ]
  @last_request = msg

  # connecting to the SIB
  tcpc = TCPConnection.new(@ip, @port)
  
  # sending the request  message
  tcpc.send_request(msg)

  # waiting for a reply
  rmsg = tcpc.receive_reply()

  # closing the socket
  tcpc.close()

  # storing last reply
  @last_reply = rmsg

  # reading message
  r = ReplyMessage.new(rmsg)

  # increment transaction id
  @transaction_id += 1

  # return
  return r.success?()
  
end

#leave_ssObject

LEAVE



159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
# File 'lib/ruby_kpi.rb', line 159

def leave_ss()

  # debug print
  if @debug
    @logger.debug("KP:leave_ss")
  end

  # building and storing the SSAP LEAVE REQUEST
  msg = LEAVE_REQUEST_TEMPLATE % [ @node_id, @ss, @transaction_id ]
  @last_request = msg

  # connecting to the SIB
  tcpc = TCPConnection.new(@ip, @port)
  
  # sendind the request
  tcpc.send_request(msg)

  # waiting a reply
  rmsg = tcpc.receive_reply()

  ## instantiate a new ReplyMessage
  r = ReplyMessage.new(rmsg)

  # closing the socket
  tcpc.close()

  # storing last reply
  @last_reply = rmsg

  # increment transaction id
  @transaction_id += 1

  # return
  return r.success?()
  
end

#rdf_indication_receiver(tcpc, subscription_id, handler) ⇒ Object

rdf indication receiver



628
629
630
631
632
633
634
635
636
637
638
639
640
641
642
643
644
645
646
647
648
649
650
651
652
653
654
655
656
657
658
659
660
661
662
663
664
665
666
667
668
669
670
671
672
673
674
675
676
677
678
679
680
# File 'lib/ruby_kpi.rb', line 628

def rdf_indication_receiver(tcpc, subscription_id, handler)

  # debug print
  if @debug
    @logger.debug("KP:rdf_indication_receiver")
  end

  # Endless loop
  while true
    
    # receive
    rmsg = tcpc.receive_reply()
    r = ReplyMessage.new(rmsg)

    # is it an indication?
    if r.get_message_type() == "INDICATION"

      # debug print
      if @debug
        @logger.debug("KP:rdf_indication_receiver -- INDICATION")
      end

      # call the handler
      added, removed = r.get_rdf_triples_from_indication()
      handler.handle(added, removed)          
      
      # it is an unsubscribe confirm
    else

      # close subscription
      if r.success?()

        # debug print
        if @debug
          @logger.debug("KP:rdf_indication_receiver -- UNSUBSCRIBE CONFIRM")
        end

        # save the reply
        @last_reply = rmsg
        
        # close subscription
        @active_subscriptions[subscription_id]["socket"].close()
        t = @active_subscriptions[subscription_id]["thread"]
        @active_subscriptions.delete(subscription_id)
        
        # return 
        r = ReplyMessage.new(rmsg)
        return r.success?()
        
      end
    end 
  end    
end

#rdf_query(triple) ⇒ Object

RDF QUERY



376
377
378
379
380
381
382
383
384
385
386
387
388
389
390
391
392
393
394
395
396
397
398
399
400
401
402
403
404
405
406
407
408
409
410
411
412
413
414
415
416
# File 'lib/ruby_kpi.rb', line 376

def rdf_query(triple)

  # debug print
  if @debug
    @logger.debug("KP:rdf_query")
  end

  # build the triple
  triple_string = TRIPLE_TEMPLATE % [triple.subject.class.to_s.split("::").last.downcase, 
                                     triple.subject.value, 
                                     triple.predicate.value, 
                                     triple.object.class.to_s.split("::").last.downcase, 
                                     triple.object.value]
  
  # build and store the SSAP QUERY REQUEST
  msg = RDF_QUERY_REQUEST_TEMPLATE % [ @node_id, @ss, @transaction_id, triple_string ]
  @last_request = msg

  # connecting to the SIB
  tcpc = TCPConnection.new(@ip, @port)
  
  # sendind the request
  tcpc.send_request(msg)

  # waiting a reply
  rmsg = tcpc.receive_reply()

  # storing and reading the reply
  @last_reply = rmsg
  r = ReplyMessage.new(rmsg)
  
  # closing the socket
  tcpc.close()

  # increment transaction id
  @transaction_id += 1

  # return
  return r.success?(), r.get_rdf_triples()

end

#rdf_subscribe(triple, myHandlerClass) ⇒ Object

RDF Subscription



470
471
472
473
474
475
476
477
478
479
480
481
482
483
484
485
486
487
488
489
490
491
492
493
494
495
496
497
498
499
500
501
502
503
504
505
506
507
508
509
510
511
512
513
514
515
516
517
518
519
520
521
522
523
524
# File 'lib/ruby_kpi.rb', line 470

def rdf_subscribe(triple, myHandlerClass)

  # debug print
  if @debug
    @logger.debug("KP:rdf_subscribe")
  end

  # build the triple
  triple_string = TRIPLE_TEMPLATE % [triple.subject.class.to_s.split("::").last.downcase, 
                                     triple.subject.value, 
                                     triple.predicate.value, 
                                     triple.object.class.to_s.split("::").last.downcase, 
                                     triple.object.value]

  # build and store the SSAP QUERY REQUEST
  msg = RDF_SUBSCRIBE_REQUEST_TEMPLATE % [ @node_id, @ss, @transaction_id, triple_string ]
  @last_request = msg

  # connecting to the SIB
  tcpc = TCPConnection.new(@ip, @port)
  
  # sendind the request
  tcpc.send_request(msg)

  # waiting a reply
  rmsg = tcpc.receive_reply()

  # storing and reading the last reply
  @last_reply = rmsg
  r = ReplyMessage.new(rmsg)

  # increment transaction id
  @transaction_id += 1

  # get the subscription id
  subscription_id = r.get_subscription_id()

  # Get the initial results
  triple_list = r.get_rdf_triples()

  # instantiate the handler class
  h = myHandlerClass.new()

  # start the thread
  t = Thread.new{rdf_indication_receiver(tcpc, subscription_id, h)}    

  # store the subscription id, its socket and its thread
  @active_subscriptions[subscription_id] = {} 
  @active_subscriptions[subscription_id]["socket"] = tcpc
  @active_subscriptions[subscription_id]["thread"] = t

  # return
  return r.success?(), subscription_id, triple_list
  
end

#remove(triple_list) ⇒ Object

REMOVE



261
262
263
264
265
266
267
268
269
270
271
272
273
274
275
276
277
278
279
280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
295
296
297
298
299
300
301
302
303
304
# File 'lib/ruby_kpi.rb', line 261

def remove(triple_list)

  # debug print
  if @debug
    @logger.debug("KP:remove")
  end

  # build the triple string
  triple_string = ""
  triple_list.each do |triple|
    triple_string += TRIPLE_TEMPLATE % [triple.subject.class.to_s.split("::").last.downcase, 
                                        triple.subject.value, 
                                        triple.predicate.value, 
                                        triple.object.class.to_s.split("::").last.downcase, 
                                        triple.object.value]
  end

  # building and storing the SSAP REMOVE REQUEST
  msg = REMOVE_REQUEST_TEMPLATE % [ @node_id, @ss, @transaction_id, triple_string ]
  @last_request = msg

  # connecting to the SIB
  tcpc = TCPConnection.new(@ip, @port)
  
  # sendind the request
  tcpc.send_request(msg)

  # waiting a reply
  rmsg = tcpc.receive_reply()

  # closing the socket
  tcpc.close()

  # storing last reply
  @last_reply = rmsg

  # increment transaction id
  @transaction_id += 1

  # parsing the message to get the return value
  r = ReplyMessage.new(rmsg)
  return r.success?()
  
end

#sparql_indication_receiver(tcpc, subscription_id, handler) ⇒ Object

sparql indication receiver



689
690
691
692
693
694
695
696
697
698
699
700
701
702
703
704
705
706
707
708
709
710
711
712
713
714
715
716
717
718
719
720
721
722
723
724
725
726
727
728
729
730
731
732
733
734
735
736
737
738
739
740
741
742
743
744
# File 'lib/ruby_kpi.rb', line 689

def sparql_indication_receiver(tcpc, subscription_id, handler)

  # debug print
  if @debug
    @logger.debug("KP:sparql_indication_receiver")
  end

  # Endless loop
  while true
    
    # receive
    rmsg = tcpc.receive_reply()
    r = ReplyMessage.new(rmsg)

    # parse the message
    content = XML::Parser.string(rmsg).parse
    
    # is it an indication?
    if r.get_message_type() == "INDICATION"

      # debug print
      if @debug
        @logger.debug("KP:sparql_indication_receiver -- INDICATION")
      end
      
      # extract triples from the indication and launch the handler
      added, removed = r.get_sparql_results_from_indication()
      handler.handle(added, removed)          
      
      # it is an unsubscribe confirm
    else

      # close subscription
      if r.success?()

        # debug print
        if @debug
          @logger.debug("KP:sparql_indication_receiver -- UNSUBSCRIBE CONFIRM")
        end
        
        # save the reply
        @last_reply = rmsg
        
        # close subscription
        @active_subscriptions[subscription_id]["socket"].close()
        t = @active_subscriptions[subscription_id]["thread"]
        @active_subscriptions.delete(subscription_id)

        # return 
        r = ReplyMessage.new(rmsg)
        return r.success?()
        
      end
    end 
  end    
end

#sparql_query(q) ⇒ Object

SPARQL QUERY



425
426
427
428
429
430
431
432
433
434
435
436
437
438
439
440
441
442
443
444
445
446
447
448
449
450
451
452
453
454
455
456
457
458
459
460
461
# File 'lib/ruby_kpi.rb', line 425

def sparql_query(q)

  # debug print
  if @debug
    @logger.debug("KP:sparql_query")
  end

  # build and store the SSAP SPARQL QUERY REQUEST
  q = q.gsub("<", "&lt;").gsub(">", "&gt;")
  msg = SPARQL_QUERY_REQUEST_TEMPLATE % [ @node_id, @ss, @transaction_id, q ]
  @last_request = msg
  
  # connecting to the SIB
  tcpc = TCPConnection.new(@ip, @port)
  
  # sendind the request
  tcpc.send_request(msg)

  # waiting a reply
  rmsg = tcpc.receive_reply()

  # closing the socket
  tcpc.close()

  # storing last reply
  @last_reply = rmsg

  # increment transaction id
  @transaction_id += 1
  
  # reading the reply
  r = ReplyMessage.new(rmsg)

  # return
  return r.success?(), r.get_sparql_results()
  
end

#sparql_subscribe(pattern, myHandlerClass = nil) ⇒ Object

SPARQL Subscription



533
534
535
536
537
538
539
540
541
542
543
544
545
546
547
548
549
550
551
552
553
554
555
556
557
558
559
560
561
562
563
564
565
566
567
568
569
570
571
572
573
574
575
576
577
578
579
580
581
582
583
584
# File 'lib/ruby_kpi.rb', line 533

def sparql_subscribe(pattern, myHandlerClass = nil)

  # debug print
  if @debug
    @logger.debug("KP:rdf_subscribe")
  end

  # build and store the SSAP QUERY REQUEST
  msg = SPARQL_SUBSCRIBE_REQUEST_TEMPLATE % [ @node_id, @ss, @transaction_id, pattern ]
  @last_request = msg

  # connecting to the SIB
  tcpc = TCPConnection.new(@ip, @port)
  
  # sendind the request
  tcpc.send_request(msg)

  # waiting a reply
  rmsg = tcpc.receive_reply()

  # storing and reading the last reply
  @last_reply = rmsg
  r = ReplyMessage.new(rmsg)

  # increment transaction id
  @transaction_id += 1

  # get the subscription id
  subscription_id = r.get_subscription_id()

  # Get the initial results
  initial_results = r.get_sparql_results()

  # start the indication receiver
  if myHandlerClass
    # TODO check if myHandlerClass is a Handler,
    # otherwise raise an exception
    h = myHandlerClass.new()
  else
    h = nil
  end
  t = Thread.new{sparql_indication_receiver(tcpc, subscription_id, h)}    

  # store the subscription id and its socket
  @active_subscriptions[subscription_id] = {} 
  @active_subscriptions[subscription_id]["socket"] = tcpc
  @active_subscriptions[subscription_id]["thread"] = t

  # return
  return r.success?(), subscription_id, initial_results
  
end

#unsubscribe(sub_id) ⇒ Object

unsubscribe



593
594
595
596
597
598
599
600
601
602
603
604
605
606
607
608
609
610
611
612
613
614
615
616
617
618
619
# File 'lib/ruby_kpi.rb', line 593

def unsubscribe(sub_id)

  # debug print
  if @debug
    @logger.debug("KP:unsubscribe")
  end

  # building and storing the SSAP UNSUBSCRIBE REQUEST
  msg = UNSUBSCRIBE_REQUEST_TEMPLATE % [ @node_id, @ss, @transaction_id, sub_id ]
  @last_request = msg

  # connecting to the SIB
  tcpc = TCPConnection.new(@ip, @port)
  
  # sending the request  message
  tcpc.send_request(msg)

  # closing the socket
  tcpc.close()

  # increment transaction id
  @transaction_id += 1
  
  # get the thread and return
  return @active_subscriptions[sub_id]["thread"].value

end

#update(new_triple_list, old_triple_list) ⇒ Object

UPDATE



313
314
315
316
317
318
319
320
321
322
323
324
325
326
327
328
329
330
331
332
333
334
335
336
337
338
339
340
341
342
343
344
345
346
347
348
349
350
351
352
353
354
355
356
357
358
359
360
361
362
363
364
365
366
367
# File 'lib/ruby_kpi.rb', line 313

def update(new_triple_list, old_triple_list)

  # debug print
  if @debug
    @logger.debug("KP:update")
  end

  # build the triple strings
  old_triple_string = ""
  old_triple_list.each do |triple|

    old_triple_string += TRIPLE_TEMPLATE % [triple.subject.class.to_s.split("::").last.downcase, 
                                            triple.subject.value, 
                                            triple.predicate.value, 
                                            triple.object.class.to_s.split("::").last.downcase, 
                                            triple.object.value]
    
  end
  new_triple_string = ""
  new_triple_list.each do |triple|

    new_triple_string += TRIPLE_TEMPLATE % [triple.subject.class.to_s.split("::").last.downcase, 
                                            triple.subject.value, 
                                            triple.predicate.value, 
                                            triple.object.class.to_s.split("::").last.downcase, 
                                            triple.object.value]
  end

  # building and storing the SSAP REMOVE REQUEST
  msg = UPDATE_REQUEST_TEMPLATE % [ @node_id, @ss, @transaction_id, new_triple_string, old_triple_string ]
  @last_request = msg

  # connecting to the SIB
  tcpc = TCPConnection.new(@ip, @port)
  
  # sendind the request
  tcpc.send_request(msg)

  # waiting a reply
  rmsg = tcpc.receive_reply()

  # closing the socket
  tcpc.close()

  # storing last reply
  @last_reply = rmsg

  # increment transaction id
  @transaction_id += 1

  # parsing the message to get the return value
  r = ReplyMessage.new(rmsg)
  return r.success?()
  
end